package com.sunxung.factoring.service.bocom.api.impl;

import com.alibaba.fastjson.JSON;
import com.bocom.api.*;
import com.bocom.api.security.spi.impl.NormalLevelSignApi;
import com.bocom.api.utils.ApiUtils;
import com.bocom.api.utils.FileItem;
import com.bocom.api.utils.enums.EncryptType;
import com.sunxung.factoring.component.constance.BocomApiConstant;
import com.sunxung.factoring.entity.bocom.log.BocomRequestLog;
import com.sunxung.factoring.entity.bocom.request.FileUploadRequestV1;
import com.sunxung.factoring.entity.bocom.response.FileUploadResponseV1;
import com.sunxung.factoring.service.bocom.api.IBocomApiService;
import com.sunxung.factoring.service.bocom.log.IBocomRequestLogService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.HashMap;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;

/**
 * @Description : 交通银行API接口实现类
 * @Author : L.Jia
 * @Date: 2024-07-29 11:48
 */
@Service
public class BocomApiServiceImpl implements IBocomApiService {

    private static final Logger logger = LoggerFactory.getLogger(BocomApiServiceImpl.class);

    /**
     * APP_ID
     */
    @Value("${bocom.appId}")
    private String APP_ID;

    /**
     * 第三方私钥
     */
    @Value("${bocom.myPrivateKey}")
    private String MY_PRIVATE_KEY;

    /**
     * 交行公钥
     */
    @Value("${bocom.apigwPublicKey}")
    private String APIGW_PUBLIC_KEY;

    /**
     * 调用地址
     */
    @Value("${bocom.apigwUrlAddress}")
    private String APIGW_URL_ADDRESS;

    @Autowired
    private IBocomRequestLogService bocomRequestLogService;


    public DefaultBocomClient getClient() {
        DefaultBocomClient client = new DefaultBocomClient(APP_ID, MY_PRIVATE_KEY, APIGW_PUBLIC_KEY);
        //测试环境可以忽略SSL证书告警，生产环境不可忽略(需要注释掉这个代码)
        client.ignoreSSLHostnameVerifier();
        return client;
    }

    public DefaultBocomClient getSpiClient() {
        //DefaultBocomClient client = new DefaultBocomClient(APP_ID, new OpenBankSM2SignatureSPI(), MY_PRIVATE_KEY, APIGW_PUBLIC_KEY);

        DefaultBocomClient client = new DefaultBocomClient(APP_ID, new NormalLevelSignApi(MY_PRIVATE_KEY), APIGW_PUBLIC_KEY);
        //测试环境可以忽略SSL证书告警，生产环境不可忽略(需要注释掉这个代码)
        client.ignoreSSLHostnameVerifier();
        return client;
    }

    @Override
    public FileUploadResponseV1 fileUploadV1(File file) {
        try {
            DefaultBocomClient client = new DefaultBocomClient(APP_ID, MY_PRIVATE_KEY, BocomConstants.CHARSET_UTF8,
                    BocomConstants.FMT_FILE, APIGW_PUBLIC_KEY);
            //测试环境可以忽略SSL证书告警，生产环境不可忽略(需要注释掉这个代码)
            client.ignoreSSLHostnameVerifier();

            FileUploadRequestV1 request = new FileUploadRequestV1();
            request.setServiceUrl(APIGW_URL_ADDRESS + BocomApiConstant.FILE_UPLOAD_URL);
            request.setFileItem(new FileItem(file));
            logger.error("请求交通银行request：" + JSON.toJSONString(request));
            FileUploadResponseV1 response = client.execute(request, UUID.randomUUID().toString().replace("-", ""));

            //添加日志
            BocomRequestLog requestLog = new BocomRequestLog();
            requestLog.setUrl(BocomApiConstant.FILE_UPLOAD_URL);
            //文件content太长，日志剔除掉
            FileItem fileItem = new FileItem(file);
            byte[] byteArray = new byte[0];
            fileItem.setContent(byteArray);
            request.setFileItem(fileItem);
            requestLog.setRequest(JSON.toJSONString(request));
            requestLog.setResponse(JSON.toJSONString(response));
            requestLog.setSuccess(response.isSuccess());
            // 日志采用异步保存，否则可能因为调用方法添加事务导致异常后的回滚
            CompletableFuture.runAsync(() -> bocomRequestLogService.save(requestLog));
            return response;
        } catch (Exception xcp) {
            xcp.printStackTrace();
        }
        return null;
    }

    @Override
    public <T extends BocomResponse> T sendToBocom(BocomRequest<T> request, String url) {
        try {
            DefaultBocomClient client;
            if (url.equals(BocomApiConstant.B2BJEBM_1001_URL) || url.equals(BocomApiConstant.B2BJEBM_1005_URL)
                    || url.equals(BocomApiConstant.B2BJEBM_1007_URL)) {
                client = getSpiClient();
            } else {
                client = getClient();
            }
            request.setServiceUrl(APIGW_URL_ADDRESS + url);
            logger.error("请求交通银行request：" + JSON.toJSONString(request));
            T t = client.execute(request, UUID.randomUUID().toString().replace("-", ""));
            //添加日志
            BocomRequestLog requestLog = new BocomRequestLog();
            requestLog.setUrl(url);
            requestLog.setRequest(JSON.toJSONString(request));
            requestLog.setResponse(JSON.toJSONString(t));
            requestLog.setSuccess(t.isSuccess());

            // 日志采用异步保存，否则可能因为调用方法添加事务导致异常后的回滚
            CompletableFuture.runAsync(() -> bocomRequestLogService.save(requestLog));
            return t;
        } catch (Exception xcp) {
            xcp.printStackTrace();
        }
        return null;
    }

    @Override
    public HashMap<String, String> decryptRequest(String request) {
        try {
            logger.error("交通银行回调接口请求报文：" + request);
            HashMap<String, String> requestMap = ApiUtils.parseCommunicationJsonWithBocomSign(EncryptType.SM2_AND_SM4, request, MY_PRIVATE_KEY, APIGW_PUBLIC_KEY);
            return requestMap;
        } catch (BocomApiException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public String encryptResponse(String response, String encryptKey) {

        try {
            String responseMsg = ApiUtils.signRspBizContent(EncryptType.SM2_AND_SM4, response, encryptKey, MY_PRIVATE_KEY);
            logger.error("交通银行回调接口响应报文：" + responseMsg);
            return responseMsg;
        } catch (BocomApiException e) {
            e.printStackTrace();
        }
        return null;
    }
}
